@@ -127,6 +127,7 @@ static void         my_add_text_and_entity     (TextRenderCtx *xargs, GString *b
 static void         my_populate_tree_store     (TreeRenderCtx *xargs, xmlNode *node, GtkTreeIter *parent, gint pos);
 
 static void         my_XML_DOCUMENT_NODE       (TextRenderCtx *xargs, xmlNode *node);
+static void         my_XML_HTML_DOCUMENT_NODE  (TextRenderCtx *xargs, xmlNode *node);
 static void         my_XML_ELEMENT_NODE        (TextRenderCtx *xargs, xmlNode *node);
 static void         my_XML_ATTRIBUTE_NODE      (TextRenderCtx *xargs, xmlNode *node);
 static void         my_XML_ATTRIBUTE_VALUE     (TextRenderCtx *xargs, xmlNode *node);
@@ -316,7 +317,6 @@ void xacobeo_populate_gtk_text_buffer (GtkTextBuffer *buffer, xmlNode *node, HV
 		WARN("GtkTextBuffer is NULL");
 		return;
 	}
-
 	
 	TextRenderCtx xargs = {
 		.buffer = buffer,
@@ -414,12 +414,15 @@ static void my_display_document_syntax (TextRenderCtx *xargs, xmlNode *node) {
 	}
 	
 	switch (node->type) {
-			buffer_add(xargs, xargs->markup->syntax, "\n");
 
 		case XML_DOCUMENT_NODE:
 			my_XML_DOCUMENT_NODE(xargs, node);
 		break;
 
+		case XML_HTML_DOCUMENT_NODE:
+			my_XML_HTML_DOCUMENT_NODE(xargs, node);
+		break;
+
 		case XML_ELEMENT_NODE:
 			my_XML_ELEMENT_NODE(xargs, node);
 		break;
@@ -453,7 +456,7 @@ static void my_display_document_syntax (TextRenderCtx *xargs, xmlNode *node) {
 		break;
 		
 		default:
-			WARN("Unknown XML type %d for %s = %s", node->type, node->name, node->content);
+			WARN("Unknown XML type %d for %s", node->type, node->name);
 		break;
 	}
 }
@@ -491,6 +494,20 @@ static void my_XML_DOCUMENT_NODE (TextRenderCtx *xargs, xmlNode *node) {
 
 
 
+// Displays an HTML 'Document' node.
+static void my_XML_HTML_DOCUMENT_NODE (TextRenderCtx *xargs, xmlNode *node) {
+	for (xmlNode *child = node->children; child; child = child->next) {
+		my_display_document_syntax(xargs, child);
+		// Add some new lines between the elements of the prolog. Libxml removes
+		// the white spaces in the prolog.
+		if (child != node->last) {
+			buffer_add(xargs, xargs->markup->syntax, "\n");
+		}
+	}
+}
+
+
+
 // Displays an Element ex: <tag>...</tag>
 static void my_XML_ELEMENT_NODE (TextRenderCtx *xargs, xmlNode *node) {
 
@@ -817,26 +834,22 @@ static gchar* my_get_node_name_prefixed (xmlNode *node, HV *namespaces) {
 //
 static const gchar* my_get_uri_prefix (const xmlChar *uri, HV *namespaces) {
 
-	const gchar *prefix = NULL;
-	
 	// Get the prefix corresponding to the namespace
 	SV **svPtr = hv_fetch(namespaces, (gchar *) uri, xmlStrlen(uri), FALSE);
-	if (svPtr) {
-		if (SvTYPE(*svPtr) == SVt_PV) {
-			// Ok found the prefix!
-			prefix = SvPVX(*svPtr);
-		}
-		else {
-			// Prefix isn't a string, something else was stored in the hash
-			WARN("No valid namespace associated with URI %s", uri);
-		}
-	}
-	else {
+	if (!svPtr) {
 		// Can't find the prefix of the URI
 		WARN("Can't find namespace for URI %s", uri);
+		return NULL;
+	}
+	
+	if (SvTYPE(*svPtr) != SVt_PV) {
+		// Prefix isn't a string, something else was stored in the hash
+		WARN("No valid namespace associated with URI %s, got: '%s'", uri, SvPV_nolen(*svPtr));
+		return NULL;
 	}
 
-	return prefix;
+	// Ok found the prefix!
+	return SvPVX(*svPtr);
 }
 
 
@@ -883,12 +896,14 @@ static gchar* my_to_string (xmlNode *node) {
 //
 static void my_buffer_add (TextRenderCtx *xargs, GtkTextTag *tag, const gchar *text) {
 
+	const gchar *content = text ? text : "";
+
 	++xargs->calls;
-	g_string_append(xargs->xml_data, text);
+	g_string_append(xargs->xml_data, content);
 	
 	// We don't want the length of the string but the number of characters.
 	// UTF-8 may encode one character as multiple bytes.
-	glong end = xargs->buffer_pos + g_utf8_strlen(text, -1);
+	glong end = xargs->buffer_pos + g_utf8_strlen(content, -1);
 
 	// Apply the markup if there's a tag
 	if (tag) {
